// src/testsuite/test-python_loader.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

/* This executable tests for workingness of the PythonPolicyManager
 * class and its closely related cousins. More documentation to be written
 * here, thanks very much. */

#include "../python_policy_manager.hh"
#include "../python_policy.hh"

#include "global_preferences.hh"
#include "ready_queue.hh"
#include "scheduler.hh"
#include "user_interrupt_exception.hh"

#include <Python.h>
#include <glibmm/module.h>

#include <cassert>
#include <iostream>
#include <string>

// FIXME: Eeeeh? Why does this work without explicit namespace resolving?
// Is there some using declaration in included HEADERS?? Aaaaagh!

class TestPythonPolicyManager : public PythonPolicyManager
{
public:
  void test_init(const char* policy_name)
  {
    init();
    _python_policy = new PythonPolicy(policy_name);

  }

  Policy& get_policy()
  {
    return *_python_policy;
  }
};


int
main(int argc, char** argv)
{
  using namespace sgpem;
  using namespace std;

  int successes = 0;

  if(argc != 2)
  {
    std::cout << "[EE] Usage:\n\t" << argv[0] <<
    " path/to/uninstalled/policies" << std::endl;
    exit(-1);
  }
  else
    // Add argv[1] as the directory to search for uninstalled policies
    sgpem::GlobalPreferences::get_instance().add_policies_dir(argv[1]);

  // Self-register itself to Scheduler, however we don't care about it
  TestPythonPolicyManager polman;

  try
  {
    polman.test_init("python_loader_configure");
    polman.get_policy().configure();
  }
  catch(UserInterruptException e)
  {
    cout << "configure: Caught UserInterruptException" << endl;
    successes++;
  }


  try
  {
    polman.test_init("python_loader_is_preemptive");
    polman.get_policy().is_pre_emptive();
  }
  catch(UserInterruptException e)
  {
    cout << "is_preemptive: Caught UserInterruptException" << endl;
    successes++;
  }

  try
  {
    polman.test_init("python_loader_get_time_slice");
    polman.get_policy().get_time_slice();
  }
  catch(UserInterruptException e)
  {
    cout << "get_time_slice: Caught UserInterruptException" << endl;
    successes++;
  }


  try
  {
    ReadyQueue sl;
    polman.test_init("python_loader_sort_queue");
    polman.get_policy().sort_queue();
  }
  catch(UserInterruptException e)
  {
    cout << "sort_queue: Caught UserInterruptException" << endl;
    successes++;
  }

  cout << "Result: catched " << successes
  << " exceptions out of 4." << endl;

  exit(0);
}
